require 'rails/generators/base'

module NexusCqrs
  class CommandGenerator < Rails::Generators::NamedBase
    source_root File.expand_path('templates', __dir__)

    def copy_command_file
      file_path = class_name.underscore

      template('command.rb', "app/domain/commands/#{file_path}.rb")
      template('command_handler.rb', "app/domain/commands/#{file_path}_handler.rb")

      register_command(class_name)
    end

    private

    def register_command(full_name)
      handler_config = 'config/initializers/register_cqrs_handlers.rb'

      unless File.exist?('config/initializers/register_cqrs_handlers.rb')
        template('register_cqrs_handlers.rb', handler_config)
      end

      code_to_inject = "$COMMAND_EXECUTOR.register_command(#{full_name}, #{full_name}Handler.new)\n"

      inject_into_file(handler_config, code_to_inject, after: "# Register Commands\n")
    end
  end
end
