require 'rails/generators/base'

module NexusCqrs
  class QueryGenerator < Rails::Generators::NamedBase
    source_root File.expand_path('templates', __dir__)

    def copy_query_file
      file_path = class_name.underscore

      template('query.rb', "app/domain/queries/#{file_path}.rb")
      template('query_handler.rb', "app/domain/queries/#{file_path}_handler.rb")

      register_query(class_name)
    end

    private

    def register_query(full_name)
      handler_config = 'config/initializers/register_cqrs_handlers.rb'

      unless File.exist?('config/initializers/register_cqrs_handlers.rb')
        template('register_cqrs_handlers.rb', handler_config)
      end

      code_to_inject = "$QUERY_EXECUTOR.register_command(#{full_name}, #{full_name}Handler.new)\n"

      inject_into_file(handler_config, code_to_inject, after: "# Register Queries\n")
    end
  end
end
