require 'thread_safe'

module NexusCqrs
  class CommandBus
    UnregisteredHandler = Class.new(StandardError)
    MultipleHandlers    = Class.new(StandardError)

    def initialize
      @handlers =
        ThreadSafe::Cache.new
    end

    def register(klass, handler)
      raise MultipleHandlers, "Multiple handlers not allowed for #{klass}" if handlers[klass]

      handlers[klass] = handler
    end

    def call(command)
      unregistered_handler = proc { raise UnregisteredHandler, "Missing handler for #{command.class}" }

      handlers
        .fetch(command.class, &unregistered_handler)
        .call(command)
    end

    def registered_handlers
      handlers
    end

    private

    attr_reader :handlers
  end
end
